@@ -13,17 +13,22 @@
 #include <gio/gdesktopappinfo.h>
 
 #include <iostream>
+#include <memory>
 
 #include "Helpers.hpp"
 #include "Store.tpp"
 
 struct AppInfo
 {
+	const std::string id;
 	const std::string path;
 	const std::string icon;
 	const std::string name;
-	const gchar* const* actions;
+	const Store::AutoPtr<GDesktopAppInfo> gAppInfo;
 
+	AppInfo(std::string _id, std::string _path, std::string _icon, std::string _name, GDesktopAppInfo* _gAppInfo = nullptr)
+		: id(_id), path(_path), icon(_icon), name(_name), gAppInfo(_gAppInfo, [](gpointer o) { if (o) g_object_unref(o); }) {}
+	const gchar* const* get_actions() { return gAppInfo ? g_desktop_app_info_list_actions(gAppInfo.get()) : nullptr; };
 	void launch();
 	void launch_action(const gchar* action);
 	void edit();
@@ -32,7 +37,8 @@ struct AppInfo
 namespace AppInfos
 {
 	void init();
-	AppInfo* search(std::string id);
+	void finalize();
+	std::shared_ptr<AppInfo> search(std::string id);
 } // namespace AppInfos
 
 #endif // APPINFOS_HPP
