@@ -10,7 +10,7 @@
 namespace Dock
 {
 	GtkWidget* mBox;
-	Store::KeyStore<AppInfo*, Group*> mGroups;
+	Store::KeyStore<std::shared_ptr<AppInfo>, std::shared_ptr<Group>> mGroups;
 
 	int mPanelSize;
 	int mIconSize;
@@ -27,18 +27,18 @@ namespace Dock
 		drawGroups();
 	}
 
-	Group* prepareGroup(AppInfo* appInfo)
+	Group* prepareGroup(std::shared_ptr<AppInfo> appInfo)
 	{
-		Group* group = mGroups.get(appInfo);
+		std::shared_ptr<Group> group = mGroups.get(appInfo);
 
-		if (group == NULL)
+		if (!group)
 		{
-			group = new Group(appInfo, false);
+			group = std::make_shared<Group>(appInfo, false);
 			mGroups.push(appInfo, group);
 			gtk_container_add(GTK_CONTAINER(mBox), group->mButton);
 		}
 
-		return group;
+		return group.get();
 	}
 
 	void moveButton(Group* moving, Group* dest)
@@ -57,29 +57,26 @@ namespace Dock
 	void savePinned()
 	{
 		std::list<std::string> pinnedList;
+		GList* children = gtk_container_get_children(GTK_CONTAINER(mBox));
 
-		for (GList* child = gtk_container_get_children(GTK_CONTAINER(mBox));
-			 child != NULL;
-			 child = child->next)
+		for (GList* child = children; child != nullptr; child = child->next)
 		{
 			GtkWidget* widget = (GtkWidget*)child->data;
 			Group* group = (Group*)g_object_get_data(G_OBJECT(widget), "group");
 
 			if (group->mPinned && g_file_test(group->mAppInfo->path.c_str(), G_FILE_TEST_IS_REGULAR))
-				pinnedList.push_back(group->mAppInfo->path);
+				pinnedList.push_back(group->mAppInfo->id);
 		}
 
 		Settings::pinnedAppList.set(pinnedList);
+		g_list_free(children);
 	}
 
 	void drawGroups()
 	{
 		// Remove old groups
-		mGroups.forEach([](std::pair<AppInfo*, Group*> g) -> void
-			{ gtk_widget_destroy(g.second->mButton); });
-
-		mGroups.clear();
 		Wnck::mGroupWindows.clear();
+		mGroups.clear();
 
 		// Add pinned groups
 		std::list<std::string> pinnedApps = Settings::pinnedAppList;
@@ -87,8 +84,8 @@ namespace Dock
 
 		while (it != pinnedApps.end())
 		{
-			AppInfo* appInfo = AppInfos::search(*it);
-			Group* group = new Group(appInfo, true);
+			std::shared_ptr<AppInfo> appInfo = AppInfos::search(Help::String::toLowercase(*it));
+			std::shared_ptr<Group> group = std::make_shared<Group>(appInfo, true);
 
 			mGroups.push(appInfo, group);
 			gtk_container_add(GTK_CONTAINER(mBox), group->mButton);
@@ -97,15 +94,15 @@ namespace Dock
 
 		// Add open windows
 		for (GList* window_l = wnck_screen_get_windows(Wnck::mWnckScreen);
-			 window_l != NULL;
+			 window_l != nullptr;
 			 window_l = window_l->next)
 		{
 			WnckWindow* wnckWindow = WNCK_WINDOW(window_l->data);
 			gulong windowXID = wnck_window_get_xid(wnckWindow);
-			GroupWindow* groupWindow = Wnck::mGroupWindows.get(windowXID);
+			std::shared_ptr<GroupWindow> groupWindow = Wnck::mGroupWindows.get(windowXID);
 
-			if (groupWindow == NULL)
-				groupWindow = new GroupWindow(wnckWindow);
+			if (!groupWindow)
+				groupWindow = std::make_shared<GroupWindow>(wnckWindow);
 			else
 				gtk_container_add(GTK_CONTAINER(mBox), groupWindow->mGroup->mButton);
 
@@ -119,38 +116,37 @@ namespace Dock
 	void hoverSupered(bool on)
 	{
 		int grabbedKeys = Hotkeys::mGrabbedKeys;
+		GList* children = gtk_container_get_children(GTK_CONTAINER(mBox));
 
-		for (GList* child = gtk_container_get_children(GTK_CONTAINER(mBox));
-			 child != NULL && grabbedKeys;
-			 child = child->next)
+		for (GList* child = children; child != nullptr && grabbedKeys; child = child->next)
 		{
 			GtkWidget* widget = (GtkWidget*)child->data;
 
 			if (!gtk_widget_get_visible(widget))
 				continue;
 
-			Group* group = (Group*)g_object_get_data(G_OBJECT(widget), "group");
-			group->mSSuper = on;
 			--grabbedKeys;
 		}
+
+		g_list_free(children);
 	}
 
 	void activateGroup(int nb, guint32 timestamp)
 	{
 		int i = 0;
+		GList* children = gtk_container_get_children(GTK_CONTAINER(mBox));
 
-		for (GList* child = gtk_container_get_children(GTK_CONTAINER(mBox));
-			 child != NULL;
-			 child = child->next)
+		for (GList* child = children; child != nullptr; child = child->next)
 		{
 			GtkWidget* widget = (GtkWidget*)child->data;
 
 			if (gtk_widget_get_visible(widget))
+			{
 				if (i == nb)
 				{
 					Group* group = (Group*)g_object_get_data(G_OBJECT(widget), "group");
 
-					if (group->mSFocus)
+					if (group->mActive)
 						group->scrollWindows(timestamp, GDK_SCROLL_DOWN);
 					else if (group->mWindowsCount > 0)
 						group->activate(timestamp);
@@ -161,7 +157,10 @@ namespace Dock
 				}
 				else
 					++i;
+			}
 		}
+
+		g_list_free(children);
 	}
 
 	void onPanelResize(int size)
@@ -187,7 +186,7 @@ namespace Dock
 				mIconSize = mPanelSize * 0.8;
 		}
 
-		mGroups.forEach([](std::pair<AppInfo*, Group*> g) -> void
+		mGroups.forEach([](std::pair<std::shared_ptr<AppInfo>, std::shared_ptr<Group>> g) -> void
 			{ g.second->resize(); });
 	}
 
