@@ -27,7 +27,7 @@ namespace Dock
 {
 	void init();
 
-	Group* prepareGroup(AppInfo* appInfo);
+	Group* prepareGroup(std::shared_ptr<AppInfo> appInfo);
 
 	void moveButton(Group* moving, Group* dest);
 	void savePinned();
@@ -40,7 +40,7 @@ namespace Dock
 	void onPanelOrientationChange(GtkOrientation orientation);
 
 	extern GtkWidget* mBox;
-	extern Store::KeyStore<AppInfo*, Group*> mGroups;
+	extern Store::KeyStore<std::shared_ptr<AppInfo>, std::shared_ptr<Group>> mGroups;
 
 	extern int mPanelSize;
 	extern int mIconSize;
