@@ -10,13 +10,13 @@
 static GtkTargetEntry entries[1] = {{(gchar*)"application/docklike_group", 0, 0}};
 static GtkTargetList* targetList = gtk_target_list_new(entries, 1);
 
-Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
+Group::Group(std::shared_ptr<AppInfo> appInfo, bool pinned) : mGroupMenu(this)
 {
-	mIconPixbuf = NULL;
+	mIconPixbuf = nullptr;
 	mAppInfo = appInfo;
 	mPinned = pinned;
 	mTopWindowIndex = 0;
-	mActive = mSFocus = mSOpened = mSMany = mSHover = mSSuper = false;
+	mActive = false;
 
 	//--------------------------------------------------
 
@@ -53,7 +53,7 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 
 	//--------------------------------------------------
 
-	mButton = gtk_button_new();
+	mButton = GTK_WIDGET(g_object_ref(gtk_button_new()));
 	mImage = gtk_image_new();
 	mLabel = gtk_label_new("");
 	GtkWidget* overlay = gtk_overlay_new();
@@ -143,7 +143,6 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 	g_signal_connect(G_OBJECT(mButton), "enter-notify-event",
 		G_CALLBACK(+[](GtkWidget* widget, GdkEventCrossing* event, Group* me) {
 			Help::Gtk::cssClassAdd(me->mButton, "hover_group");
-			me->mSHover = true;
 			me->mLeaveTimeout.stop();
 			me->mMenuShowTimeout.start();
 
@@ -159,7 +158,6 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 	g_signal_connect(G_OBJECT(mButton), "leave-notify-event",
 		G_CALLBACK(+[](GtkWidget* widget, GdkEventCrossing* event, Group* me) {
 			Help::Gtk::cssClassRemove(me->mButton, "hover_group");
-			me->mSHover = false;
 			me->mMenuShowTimeout.stop();
 
 			if (me->mPinned && !me->mWindowsCount)
@@ -188,13 +186,10 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 	if (mPinned)
 		gtk_widget_show_all(mButton);
 
-	if (mAppInfo != NULL && !mAppInfo->icon.empty())
+	if (mAppInfo != nullptr && !mAppInfo->icon.empty())
 	{
 		if (mAppInfo->icon[0] == '/' && g_file_test(mAppInfo->icon.c_str(), G_FILE_TEST_IS_REGULAR))
-		{
-			mIconPixbuf = gdk_pixbuf_new_from_file(mAppInfo->icon.c_str(), NULL);
-			gtk_image_set_from_pixbuf(GTK_IMAGE(mImage), mIconPixbuf);
-		}
+			mIconPixbuf = gdk_pixbuf_new_from_file(mAppInfo->icon.c_str(), nullptr);
 		else
 			gtk_image_set_from_icon_name(GTK_IMAGE(mImage), mAppInfo->icon.c_str(), GTK_ICON_SIZE_BUTTON);
 	}
@@ -205,6 +200,20 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 	updateStyle();
 }
 
+Group::~Group()
+{
+	mLeaveTimeout.stop();
+	mMenuShowTimeout.stop();
+
+	// can be unparented before the group is destroyed on exit
+	if (gtk_widget_get_parent(mButton) != nullptr)
+		gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(mButton)), mButton);
+	g_object_unref(mButton);
+
+	if (mIconPixbuf != nullptr)
+		g_object_unref(mIconPixbuf);
+}
+
 void Group::add(GroupWindow* window)
 {
 	mWindows.push(window);
@@ -215,6 +224,9 @@ void Group::add(GroupWindow* window)
 	if (mWindowsCount == 1 && !mPinned)
 		gtk_box_reorder_child(GTK_BOX(Dock::mBox), mButton, -1);
 
+	if (!mActive && wnck_window_is_active(window->mWnckWindow))
+		onWindowActivate(window);
+
 	gtk_widget_queue_draw(mButton);
 }
 
@@ -223,7 +235,6 @@ void Group::remove(GroupWindow* window)
 	mWindows.pop(window);
 	mWindowsCount.updateState();
 	mGroupMenu.remove(window->mGroupMenuItem);
-	mSFocus = false;
 
 	if (!mWindowsCount)
 		Help::Gtk::cssClassRemove(mButton, "open_group");
@@ -244,7 +255,6 @@ void Group::activate(guint32 timestamp)
 	});
 
 	groupWindow->activate(timestamp);
-	onWindowActivate(groupWindow);
 }
 
 void Group::scrollWindows(guint32 timestamp, GdkScrollDirection direction)
@@ -257,11 +267,11 @@ void Group::scrollWindows(guint32 timestamp, GdkScrollDirection direction)
 	else
 	{
 		if (direction == GDK_SCROLL_UP)
-			mTopWindowIndex = ++mTopWindowIndex % mWindows.size();
+			mTopWindowIndex = (mTopWindowIndex + 1) % mWindows.size();
 		else if (direction == GDK_SCROLL_DOWN)
 		{
 			int size = mWindows.size();
-			mTopWindowIndex = (--mTopWindowIndex + size) % size;
+			mTopWindowIndex = (mTopWindowIndex - 1 + size) % size;
 		}
 		mWindows.get(mTopWindowIndex)->activate(timestamp);
 	}
@@ -280,9 +290,16 @@ void Group::resize()
 	// TODO: set `min-width` / `min-height` CSS property on button?
 	// https://github.com/davekeogh/xfce4-docklike-plugin/issues/39
 	
-	if (mIconPixbuf != NULL)
-		gtk_image_set_from_pixbuf(GTK_IMAGE(mImage),
-			gdk_pixbuf_scale_simple(mIconPixbuf, Dock::mIconSize, Dock::mIconSize, GDK_INTERP_HYPER));
+	if (mIconPixbuf != nullptr)
+	{
+		gint scale_factor = gtk_widget_get_scale_factor(mButton);
+		gint size = Dock::mIconSize * scale_factor;
+		GdkPixbuf* scaled = gdk_pixbuf_scale_simple(mIconPixbuf, size, size, GDK_INTERP_BILINEAR);
+		cairo_surface_t* surface = gdk_cairo_surface_create_from_pixbuf(scaled, scale_factor, NULL);
+		gtk_image_set_from_surface(GTK_IMAGE(mImage), surface);
+		cairo_surface_destroy (surface);
+		g_object_unref(scaled);
+	}
 	else
 		gtk_image_set_pixel_size(GTK_IMAGE(mImage), Dock::mIconSize);
 
@@ -312,23 +329,20 @@ void Group::onDraw(cairo_t* cr)
 		rgba[1] = (*indicatorColor).green;
 		rgba[2] = (*indicatorColor).blue;
 		rgba[3] = (*indicatorColor).alpha;
+		g_value_unset(&gv);
 	}
 	else
 	{
-		if (mSFocus)
-		{
-			rgba[0] = (*Settings::indicatorColor).red;
-			rgba[1] = (*Settings::indicatorColor).green;
-			rgba[2] = (*Settings::indicatorColor).blue;
-			rgba[3] = (*Settings::indicatorColor).alpha;
-		}
+		std::shared_ptr<GdkRGBA> color;
+		if (mActive)
+			color = Settings::indicatorColor;
 		else
-		{
-			rgba[0] = (*Settings::inactiveColor).red;
-			rgba[1] = (*Settings::inactiveColor).green;
-			rgba[2] = (*Settings::inactiveColor).blue;
-			rgba[3] = (*Settings::inactiveColor).alpha;
-		}
+			color = Settings::inactiveColor;
+
+		rgba[0] = color->red;
+		rgba[1] = color->green;
+		rgba[2] = color->blue;
+		rgba[3] = color->alpha;
 	}
 
 	const float BAR_WEIGHT = 0.935;
@@ -361,7 +375,7 @@ void Group::onDraw(cairo_t* cr)
 	}
 	
 	int indicator_style = Settings::inactiveIndicatorStyle;
-	if (mSFocus)
+	if (mActive)
 		indicator_style = Settings::indicatorStyle;
 
 	switch (indicator_style)
@@ -371,7 +385,7 @@ void Group::onDraw(cairo_t* cr)
 
 	case STYLE_BARS:
 	{
-		if (mSOpened)
+		if (mWindowsCount > 0)
 		{
 			cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
 
@@ -387,7 +401,7 @@ void Group::onDraw(cairo_t* cr)
 			cairo_fill(cr);
 		}
 
-		if (mSMany && (mSOpened || mSHover))
+		if (mWindowsCount > 1)
 		{
 			int pat0;
 			cairo_pattern_t* pat;
@@ -425,14 +439,14 @@ void Group::onDraw(cairo_t* cr)
 
 	case STYLE_CILIORA:
 	{
-		if (mSOpened)
+		if (mWindowsCount > 0)
 		{
 		    int offset;
 			cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
 			
 			offset = 0;
 			
-			if (mSMany) {
+			if (mWindowsCount > 1) {
 			    if (orientation == ORIENTATION_BOTTOM || orientation == ORIENTATION_TOP) {
 			        offset = 2 * (round(h * (1 - BAR_WEIGHT)));
 			    } else {
@@ -452,7 +466,7 @@ void Group::onDraw(cairo_t* cr)
 			cairo_fill(cr);
 		}
 
-		if (mSMany && (mSOpened || mSHover))
+		if (mWindowsCount > 1)
 		{
 		    int size;
 			cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
@@ -480,22 +494,22 @@ void Group::onDraw(cairo_t* cr)
 
 	case STYLE_CIRCLES:
 	{
-		if (mSOpened)
+		if (mWindowsCount > 0)
 		{
-			if (mSMany)
+			if (mWindowsCount > 1)
 			{
-				double x0, y0, x1, y1, radius;
+				double x0 = 0, y0 = 0, x1 = 0, y1 = 0, radius = 0;
 
 				if (orientation == ORIENTATION_BOTTOM)
 				{
-				    radius = h * CIRCLE_WEIGHT;
+					radius = h * CIRCLE_WEIGHT;
 					x0 = (w / 2.) - radius * 1.5;
 					x1 = (w / 2.) + radius * 1.5;
 					y0 = y1 = h - radius;
 				}
 				else if (orientation == ORIENTATION_RIGHT)
 				{
-				    radius = w * CIRCLE_WEIGHT;
+					radius = w * CIRCLE_WEIGHT;
 					y0 = (h / 2.) - radius * 1.5;
 					y1 = (h / 2.) + radius * 1.5;
 					x0 = x1 = w - radius;
@@ -527,7 +541,7 @@ void Group::onDraw(cairo_t* cr)
 			}
 			else
 			{
-				double x, y, radius;
+				double x = 0, y = 0, radius = 0;
 
 				if (orientation == ORIENTATION_BOTTOM)
 				{
@@ -565,11 +579,11 @@ void Group::onDraw(cairo_t* cr)
 
 	case STYLE_DOTS:
 	{
-		if (mSOpened)
+		if (mWindowsCount > 0)
 		{
-			if (mSMany)
+			if (mWindowsCount > 1)
 			{
-				double x0, y0, x1, y1;
+				double x0 = 0, y0 = 0, x1 = 0, y1 = 0;
 
 				if (orientation == ORIENTATION_BOTTOM)
 				{
@@ -618,7 +632,7 @@ void Group::onDraw(cairo_t* cr)
 			}
 			else
 			{
-				double x, y;
+				double x = 0, y = 0;
 
 				if (orientation == ORIENTATION_BOTTOM)
 				{
@@ -657,7 +671,7 @@ void Group::onDraw(cairo_t* cr)
 
 	case STYLE_RECTS:
 	{
-		if (mSOpened)
+		if (mWindowsCount > 0)
 		{
 			int vw;
 
@@ -666,7 +680,7 @@ void Group::onDraw(cairo_t* cr)
 			else
 				vw = h;
 
-			if (mSMany)
+			if (mWindowsCount > 1)
 			{
 				int space = floor(vw / 4.5);
 				int sep = vw / 11.;
@@ -724,13 +738,12 @@ void Group::onDraw(cairo_t* cr)
 
 void Group::onMouseEnter()
 {
-	Dock::mGroups.forEach([this](std::pair<AppInfo*, Group*> g) -> void {
+	Dock::mGroups.forEach([this](std::pair<std::shared_ptr<AppInfo>, std::shared_ptr<Group>> g) -> void {
 		if (&(g.second->mGroupMenu) != &(this->mGroupMenu))
 			g.second->mGroupMenu.mGroup->onMouseLeave();
 	});
 
 	mGroupMenu.popup();
-	mSHover = true;
 }
 
 void Group::onMouseLeave()
@@ -747,37 +760,30 @@ void Group::setMouseLeaveTimeout()
 
 void Group::updateStyle()
 {
-	int wCount = mWindowsCount;
-
-	if (mPinned || wCount)
+	if (mPinned || mWindowsCount > 0)
 		gtk_widget_show_all(mButton);
 	else
 		gtk_widget_hide(mButton);
 
-	if (wCount)
+	if (mWindowsCount > 0)
 	{
-		if (wCount == 1 && Settings::noWindowsListIfSingle)
+		if (mWindowsCount == 1 && Settings::noWindowsListIfSingle)
 			gtk_widget_set_tooltip_text(mButton, mAppInfo->name.c_str());
 		else
-			gtk_widget_set_tooltip_text(mButton, NULL);
+			gtk_widget_set_tooltip_text(mButton, nullptr);
 
-		mSOpened = true;
-
-		if (wCount > 1)
-			mSMany = true;
-		else
-			mSMany = false;
-
-		if (wCount > 2 && Settings::showWindowCount)
-			gtk_label_set_markup(GTK_LABEL(mLabel), g_strdup_printf("<b>%d</b>", wCount));
+		if (mWindowsCount > 2 && Settings::showWindowCount)
+		{
+			gchar* markup = g_strdup_printf("<b>%d</b>", (int)mWindowsCount);
+			gtk_label_set_markup(GTK_LABEL(mLabel), markup);
+			g_free(markup);
+		}
 		else
 			gtk_label_set_markup(GTK_LABEL(mLabel), "");
 	}
 	else
 	{
 		gtk_widget_set_tooltip_text(mButton, mAppInfo->name.c_str());
-		mSOpened = false;
-		mSFocus = false;
 	}
 }
 
@@ -790,11 +796,11 @@ void Group::electNewTopWindow()
 		if (mWindows.size() == 1)
 			newTopWindow = mWindows.get(0);
 		else
-			newTopWindow = Wnck::mGroupWindows.findIf([this](std::pair<gulong, GroupWindow*> e) -> bool {
+			newTopWindow = Wnck::mGroupWindows.findIf([this](std::pair<gulong, std::shared_ptr<GroupWindow>> e) -> bool {
 				if (e.second->mGroup == this)
 					return true;
 				return false;
-			});
+			}).get();
 
 		setTopWindow(newTopWindow);
 	}
@@ -802,19 +808,14 @@ void Group::electNewTopWindow()
 
 void Group::onWindowActivate(GroupWindow* groupWindow)
 {
-	if (!groupWindow->getState(WNCK_WINDOW_STATE_SKIP_TASKLIST))
-	{
-		mActive = true;
-		mSFocus = true;
-		setTopWindow(groupWindow);
-		Help::Gtk::cssClassAdd(mButton, "active_group");
-	}
+	mActive = true;
+	setTopWindow(groupWindow);
+	Help::Gtk::cssClassAdd(mButton, "active_group");
 }
 
 void Group::onWindowUnactivate()
 {
 	mActive = false;
-	mSFocus = false;
 	Help::Gtk::cssClassRemove(mButton, "active_group");
 }
 
@@ -827,18 +828,20 @@ void Group::onButtonPress(GdkEventButton* event)
 {
 	if (event->button == GDK_BUTTON_SECONDARY)
 	{
-		GroupWindow* win = Wnck::mGroupWindows.findIf([this](std::pair<gulong, GroupWindow*> e) -> bool {
+		std::shared_ptr<GroupWindow> win = Wnck::mGroupWindows.findIf([this](std::pair<gulong, std::shared_ptr<GroupWindow>> e) -> bool {
 			return (e.second->mGroupAssociated && e.second->mGroup == this);
 		});
 
-		if (win == NULL && !mPinned)
+		if (!win && !mPinned)
 			return;
 
-		GtkWidget* menu = Wnck::buildActionMenu(win, this);
-
-		xfce_panel_plugin_register_menu(Plugin::mXfPlugin, GTK_MENU(menu));
-		gtk_menu_attach_to_widget(GTK_MENU(menu), mButton, NULL);
-		gtk_menu_popup_at_widget(GTK_MENU(menu), mButton, GDK_GRAVITY_SOUTH_WEST, GDK_GRAVITY_NORTH_WEST, (GdkEvent*)event);
+		if (mButton != nullptr)
+		{
+			GtkWidget* menu = GTK_WIDGET(g_object_ref_sink(Wnck::buildActionMenu(win.get(), this)));
+			xfce_panel_plugin_register_menu(Plugin::mXfPlugin, GTK_MENU(menu));
+			g_signal_connect(menu, "deactivate", G_CALLBACK(g_object_unref), nullptr);
+			gtk_menu_popup_at_widget(GTK_MENU(menu), mButton, GDK_GRAVITY_SOUTH_WEST, GDK_GRAVITY_NORTH_WEST, (GdkEvent*)event);
+		}
 
 		mGroupMenu.hide();
 	}
@@ -853,10 +856,16 @@ void Group::onButtonRelease(GdkEventButton* event)
 			case BEHAVIOR_CLOSE_ALL:
 			{
 				closeAll();
+				break;
 			}
 			case BEHAVIOR_LAUNCH_NEW:
 			{
 				mAppInfo->launch();
+				break;
+			}
+			case BEHAVIOR_DO_NOTHING:
+			{
+				break;
 			}
 		}
 	}
@@ -872,16 +881,18 @@ bool Group::onDragMotion(GtkWidget* widget, GdkDragContext* context, int x, int
 {
 	GdkModifierType mask;
 	GdkDevice* device = gdk_drag_context_get_device(context);
-	gdk_window_get_device_position(gtk_widget_get_window(widget), device, NULL, NULL, &mask);
+	gdk_window_get_device_position(gtk_widget_get_window(widget), device, nullptr, nullptr, &mask);
 
 	if (mask & GDK_CONTROL_MASK)
 		gtk_drag_cancel(context);
 
 	GList* tmp_list = gdk_drag_context_list_targets(context);
 
-	if (tmp_list != NULL)
+	if (tmp_list != nullptr)
 	{
-		std::string target = gdk_atom_name(GDK_POINTER_TO_ATOM(tmp_list->data));
+		gchar* name = gdk_atom_name(GDK_POINTER_TO_ATOM(tmp_list->data));
+		std::string target = name;
+		g_free(name);
 
 		if (target != "application/docklike_group")
 		{
@@ -921,5 +932,23 @@ void Group::onDragDataReceived(const GdkDragContext* context, int x, int y, cons
 
 void Group::onDragBegin(GdkDragContext* context)
 {
-	gtk_drag_set_icon_name(context, mAppInfo->icon.c_str(), 0, 0);
+	if (mIconPixbuf != nullptr)
+	{
+		gint scale_factor = gtk_widget_get_scale_factor(mButton);
+		gint size;
+		if (!gtk_icon_size_lookup(GTK_ICON_SIZE_DND, &size, NULL))
+			size = 32;
+		size *= scale_factor;
+		GdkPixbuf* scaled = gdk_pixbuf_scale_simple(mIconPixbuf, size, size, GDK_INTERP_BILINEAR);
+		cairo_surface_t* surface = gdk_cairo_surface_create_from_pixbuf(scaled, scale_factor, NULL);
+		gtk_drag_set_icon_surface(context, surface);
+		cairo_surface_destroy (surface);
+		g_object_unref(scaled);
+	}
+	else
+	{
+		const gchar *icon_name;
+		gtk_image_get_icon_name(GTK_IMAGE(mImage), &icon_name, nullptr);
+		gtk_drag_set_icon_name(context, icon_name, 0, 0);
+	}
 }
