@@ -11,33 +11,42 @@
 #include "GroupMenuItem.hpp"
 #include "Plugin.hpp"
 
+static GtkWidget*
+create_window()
+{
+	GtkWidget* window = gtk_window_new(GtkWindowType::GTK_WINDOW_POPUP);
+	gtk_widget_add_events(window, GDK_SCROLL_MASK);
+	gtk_window_set_default_size(GTK_WINDOW(window), 1, 1);
+	return window;
+}
+
 GroupMenu::GroupMenu(Group* dockButton)
 {
 	mGroup = dockButton;
 	mVisible = false;
 	mMouseHover = false;
-	mWindow = gtk_window_new(GtkWindowType::GTK_WINDOW_POPUP);
+	mWindow = create_window();
 	mBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	Help::Gtk::cssClassAdd(mBox, "menu");
-	gtk_widget_add_events(mWindow, GDK_SCROLL_MASK);
-	gtk_window_set_default_size(GTK_WINDOW(mWindow), 1, 1);
 	gtk_container_add(GTK_CONTAINER(mWindow), mBox);
 	gtk_widget_show(mBox);
 
+	mPopupIdle.setup([this]() {
+		popup();
+		return false;
+	});
+
 	//--------------------------------------------------
 
 	g_signal_connect(G_OBJECT(mWindow), "enter-notify-event",
 		G_CALLBACK(+[](GtkWidget* widget, GdkEvent* event, GroupMenu* me) {
 			me->mMouseHover = true;
-			me->mGroup->mSHover = true;
 
-			me->mGroup->mWindows.forEach([](GroupWindow* w) -> void {
-				gtk_widget_set_visible(GTK_WIDGET(w->mGroupMenuItem->mPreview), Settings::showPreviews);
-
-				if (Settings::showPreviews)
-					w->mGroupMenuItem->updatePreview();
-			});
+			if (Settings::showPreviews)
+				me->mGroup->mWindows.forEach([](GroupWindow* w) -> void {
+						w->mGroupMenuItem->updatePreview();
+				});
 
 			return true;
 		}),
@@ -66,11 +75,31 @@ GroupMenu::GroupMenu(Group* dockButton)
 			return true;
 		}),
 		this);
+
+	g_signal_connect(G_OBJECT(Plugin::mXfPlugin), "notify::scale-factor",
+		G_CALLBACK(+[](GtkWidget* widget, GParamSpec* pspec, GroupMenu* me) {
+			g_object_ref(me->mBox);
+			gtk_container_remove(GTK_CONTAINER(me->mWindow), me->mBox);
+			gtk_widget_destroy(me->mWindow);
+			me->mWindow = create_window();
+			gtk_container_add(GTK_CONTAINER(me->mWindow), me->mBox);
+			g_object_unref(me->mBox);
+		}),
+		this);
+}
+
+GroupMenu::~GroupMenu()
+{
+	mPopupIdle.stop();
+	gtk_widget_destroy(mWindow);
 }
 
 void GroupMenu::add(GroupMenuItem* menuItem)
 {
 	gtk_box_pack_end(GTK_BOX(mBox), GTK_WIDGET(menuItem->mItem), false, true, 0);
+
+	if (mVisible)
+		mPopupIdle.start();
 }
 
 void GroupMenu::remove(GroupMenuItem* menuItem)
@@ -80,6 +109,9 @@ void GroupMenu::remove(GroupMenuItem* menuItem)
 
 	if (mGroup->mWindowsCount < (Settings::noWindowsListIfSingle ? 2 : 1))
 		gtk_widget_hide(mWindow);
+
+	if (mVisible)
+		mPopupIdle.start();
 }
 
 void GroupMenu::popup()
@@ -93,9 +125,12 @@ void GroupMenu::popup()
 
 		// Update the previews before showing the window
 		if (Settings::showPreviews)
+		{
 			mGroup->mWindows.forEach([](GroupWindow* w) -> void {
 				w->mGroupMenuItem->updatePreview();
 			});
+			gtk_window_resize(GTK_WINDOW(mWindow), 1, 1);
+		}
 
 		xfce_panel_plugin_position_widget(Plugin::mXfPlugin, mWindow, mGroup->mButton, &wx, &wy);
 		updatePosition(wx, wy);
@@ -155,6 +190,15 @@ void GroupMenu::hide()
 	gtk_widget_hide(mWindow);
 }
 
+void GroupMenu::showPreviewsChanged()
+{
+	mGroup->mWindows.forEach([](GroupWindow* w) -> void {
+		gtk_widget_set_visible(GTK_WIDGET(w->mGroupMenuItem->mPreview), Settings::showPreviews);
+		w->mGroupMenuItem->mPreviewTimeout.stop();
+	});
+	gtk_window_resize(GTK_WINDOW(mWindow), 1, 1);
+}
+
 uint GroupMenu::getPointerDistance()
 {
 	gint wx, wy, ww, wh, px, py;
@@ -163,7 +207,7 @@ uint GroupMenu::getPointerDistance()
 
 	gtk_window_get_position(GTK_WINDOW(mWindow), &wx, &wy);
 	gtk_window_get_size(GTK_WINDOW(mWindow), &ww, &wh);
-	gdk_device_get_position(Plugin::mPointer, NULL, &px, &py);
+	gdk_device_get_position(Plugin::mPointer, nullptr, &px, &py);
 
 	if (px < wx)
 		dx = wx - px;
