@@ -12,6 +12,8 @@
 
 #include <iostream>
 
+#include "Helpers.hpp"
+
 class Group;
 class GroupMenuItem;
 
@@ -19,6 +21,7 @@ class GroupMenu
 {
   public:
 	GroupMenu(Group* dockButton);
+	~GroupMenu();
 
 	void add(GroupMenuItem* menuItem);
 	void remove(GroupMenuItem* menuItem);
@@ -27,6 +30,7 @@ class GroupMenu
 	void updateOrientation();
 	void updatePosition(gint wx, gint wy);
 	void hide();
+	void showPreviewsChanged();
 
 	uint getPointerDistance();
 
@@ -37,6 +41,8 @@ class GroupMenu
 
 	bool mVisible;
 	bool mMouseHover;
+
+	Help::Gtk::Idle mPopupIdle;
 };
 
 #endif // GROUP_MENU_HPP
